#ifndef _ENVIRONMENT_CPP
#define _ENVIRONMENT_CPP

#include "environment.h"
#include "iostream"

extern int BOX;
extern int CYLINDER;

extern int MAX_OBJECTS;
extern int MAX_JOINTS;

ENVIRONMENT::ENVIRONMENT(void) {

	objects = new OBJECT * [MAX_OBJECTS];

	joints  = new JOINT * [MAX_JOINTS];

	numberOfBodies = 0;

	numberOfJoints = 0;

	neuralNetwork = NULL;
}

ENVIRONMENT::~ENVIRONMENT(void) {

}

void ENVIRONMENT::Actuate_Joints(void) {

        for (int j=0;j<numberOfJoints;j++)

                joints[j]->Actuate();
}

void ENVIRONMENT::Draw(void) {

        for (int i=0;i<numberOfBodies;i++)

                objects[i]->Draw();
}

void ENVIRONMENT::Read_From_Python(dWorldID world,dSpaceID space, 
                                int *evaluationTime, float *dt, float *gravity,
                                float *xyz, float *hpr) {

        char incomingString[100];

        std::cin >> incomingString;

        while ( strcmp(incomingString,"Done") != 0 ) {
                //std::cerr << incomingString << "\n";
                //Simulator timing
                if ( strcmp(incomingString,"EvaluationTime") == 0 )
			std::cin >> (*evaluationTime);

                else if ( strcmp(incomingString,"TimeInterval") == 0)
                        std::cin >> (*dt);

                //Camera
                else if ( strcmp(incomingString,"Camera") == 0)
                {
                        std::cin >> xyz[0];
                        std::cin >> xyz[1];
                        std::cin >> xyz[2];

                        std::cin >> hpr[0];
                        std::cin >> hpr[1];
                        std::cin >> hpr[2];
                }

                else if ( strcmp(incomingString,"Gravity") == 0)
                        std::cin >> (*gravity);
        
                //Bodies
                else if ( strcmp(incomingString,"Box") == 0 )

			Create_Object(world,space,numberOfBodies,BOX);

		else if ( strcmp(incomingString,"Cylinder") == 0 )

			Create_Object(world,space,numberOfBodies,CYLINDER);

		else if ( strcmp(incomingString,"Joint") == 0 )

			Create_Joint(world,space,numberOfJoints);

                //Sensors
		else if ( strcmp(incomingString,"PositionSensor") == 0 )

			Create_Position_Sensor(*evaluationTime);

                else if ( strcmp(incomingString,"TouchSensor") == 0 )

                        Create_Touch_Sensor(*evaluationTime);

                else if ( strcmp(incomingString,"RaySensor") == 0 )

                        Create_Ray_Sensor(space,*evaluationTime);

                else if ( strcmp(incomingString,"ProprioceptiveSensor") == 0 )

			Create_Proprioceptive_Sensor(*evaluationTime);

                else if ( strcmp(incomingString,"LightSensor") == 0 )

                        Create_Light_Sensor(*evaluationTime);

                else if ( strcmp(incomingString,"VestibularSensor") == 0 )

                        Create_Vestibular_Sensor(*evaluationTime);

		else if ( strcmp(incomingString,"LightSource") == 0 )

			Create_Light_Source();

                //Neurons
		else if ( strcmp(incomingString,"SensorNeuron") == 0 )

			Create_Sensor_Neuron();

                else if ( strcmp(incomingString,"BiasNeuron") == 0 )

                        Create_Bias_Neuron();

		else if ( strcmp(incomingString,"HiddenNeuron") == 0 )

			Create_Hidden_Neuron();

                else if ( strcmp(incomingString,"MotorNeuron") == 0 )

                        Create_Motor_Neuron();

                else if ( strcmp(incomingString,"FunctionNeuron") == 0)
                        Create_Function_Neuron(*evaluationTime);
                //Synapse
		else
			Create_Synapse();

                std::cin >> incomingString;
        }
}

void ENVIRONMENT::Poll_Sensors(int timeStep) {

        for (int i=0;i<numberOfBodies;i++)

                objects[i]->Poll_Sensors(numberOfBodies,objects,timeStep);

        for (int j=0;j<numberOfJoints;j++)

                joints[j]->Poll_Sensors(timeStep);
}

void ENVIRONMENT::Update_Neural_Network(int timeStep) {

	Update_Sensor_Neurons(timeStep);
	if ( neuralNetwork )

		neuralNetwork->Update(timeStep);
}

void ENVIRONMENT::Write_Sensor_Data(int evalPeriod) {

	for (int i=0;i<numberOfBodies;i++)

		objects[i]->Write_To_Python(evalPeriod);

        for (int j=0;j<numberOfJoints;j++)

        	joints[j]->Write_To_Python(evalPeriod);

	std::cout << "Done\n";
}

// ----------------------- Private methods ---------------------------

void ENVIRONMENT::Add_Motor_Neuron(int ID, int jointID, double tau) {

        NEURON *motorNeuron = neuralNetwork->Add_Motor_Neuron(ID,tau);

        Connect_Motor_Neuron_to_Joint( jointID, motorNeuron );
}

void ENVIRONMENT::Add_Sensor_Neuron(int ID, int sensorID, int sensorValueIndex, double tau) {

        NEURON *sensorNeuron = neuralNetwork->Add_Sensor_Neuron(ID,sensorValueIndex,tau);

        Connect_Sensor_To_Sensor_Neuron( sensorID, sensorNeuron );
}

void ENVIRONMENT::Create_Bias_Neuron(void) {

        int ID;

        std::cin >> ID;

        if ( neuralNetwork == NULL )

                Create_Neural_Network();

        neuralNetwork->Add_Bias_Neuron(ID);
}

void ENVIRONMENT::Create_Function_Neuron(int evalPeriod){
        int ID;
        std::cin >> ID;

        if( neuralNetwork == NULL)
                Create_Neural_Network();
        double *timeValues = new double[evalPeriod];

        for(int i=0; i<evalPeriod; i++)
        {
                std::cin >> timeValues[i];
        }

        neuralNetwork->Add_Function_Neuron(ID, timeValues);
}
void ENVIRONMENT::Create_Hidden_Neuron(void) {

        int ID;

        std::cin >> ID;

	double tau;

	std::cin >> tau;

        if ( neuralNetwork == NULL )

                Create_Neural_Network();

	neuralNetwork->Add_Hidden_Neuron(ID,tau);
}

void ENVIRONMENT::Create_Joint( dWorldID world, dSpaceID space, int index) {

	joints[index] = new JOINT();

	joints[index]->Read_From_Python();

	int firstObjectID = joints[index]->Get_First_Object_Index();

	int secondObjectID = joints[index]->Get_Second_Object_Index();

	OBJECT *firstObject = NULL;

	if ( firstObjectID >= 0 )

		firstObject = objects[ firstObjectID ];

        OBJECT *secondObject = NULL;

	if ( secondObjectID >= 0 )

		secondObject = objects[ secondObjectID ];

	joints[index]->Create_In_Simulator(	world, firstObject , secondObject );

	numberOfJoints++;
}

void ENVIRONMENT::Create_Light_Sensor(int evalPeriod) {

        int objectIndex;

        int ID;

        std::cin >> ID;

        std::cin >> objectIndex;

        objects[objectIndex]->Create_Light_Sensor(ID,evalPeriod);
}

void ENVIRONMENT::Create_Light_Source(void) {

	int objectIndex;

	std::cin >> objectIndex;

	objects[objectIndex]->Create_Light_Source();
}

void ENVIRONMENT::Create_Motor_Neuron(void) {

        int ID;

        std::cin >> ID;

        int jointID;

        std::cin >> jointID;

	double tau;

	std::cin >> tau;

        if ( neuralNetwork == NULL )

                Create_Neural_Network();

        Add_Motor_Neuron(ID,jointID,tau);
}

void ENVIRONMENT::Create_Neural_Network(void) {

	neuralNetwork = new NEURAL_NETWORK();
}

void ENVIRONMENT::Create_Object(dWorldID world, dSpaceID space, int index, int shape) {

        objects[index] = new OBJECT();

	objects[index]->Read_From_Python(world,space,shape);

	numberOfBodies++;
}

void ENVIRONMENT::Create_Ray_Sensor(dSpaceID space, int evalPeriod) {

        int objectIndex;

	int ID;

	std::cin >> ID;

        std::cin >> objectIndex;

        objects[objectIndex]->Create_Ray_Sensor(space,ID,evalPeriod);
}

void ENVIRONMENT::Create_Position_Sensor(int evalPeriod) {

	int objectIndex;

	int ID;

	std::cin >> ID;

        std::cin >> objectIndex; 

	objects[objectIndex]->Create_Position_Sensor(ID,evalPeriod);	
}

void ENVIRONMENT::Create_Proprioceptive_Sensor(int evalPeriod) {

        int jointIndex;

	int ID;

	std::cin >> ID;

        std::cin >> jointIndex;

        joints[jointIndex]->Create_Proprioceptive_Sensor(ID,evalPeriod);
}

void ENVIRONMENT::Connect_Motor_Neuron_to_Joint( int jointID, NEURON *motorNeuron ) {

        int done = false;

        int jointIndex = 0;

        while ( (done == false) && (jointIndex < numberOfJoints) )

                done = joints[jointIndex++]->Connect_To_Motor_Neuron( jointID , motorNeuron );
}

void ENVIRONMENT::Connect_Sensor_To_Sensor_Neuron( int sensorID , NEURON *sensorNeuron ) {

        int done = false;

        int objectIndex = 0;

        while ( (done == false) && (objectIndex < numberOfBodies) )

                done = objects[objectIndex++]->Connect_Sensor_To_Sensor_Neuron(sensorID,sensorNeuron);

        int jointIndex = 0;

        while ( (done == false) && (jointIndex < numberOfJoints) )

                done = joints[jointIndex++]->Connect_Sensor_To_Sensor_Neuron(sensorID,sensorNeuron);
}

void ENVIRONMENT::Create_Sensor_Neuron(void) {

	int ID;

	std::cin >> ID;

	int sensorID;

        std::cin >> sensorID;

	int sensorValueIndex;

        std::cin >> sensorValueIndex;

	double tau;

	std::cin >> tau;

	if ( neuralNetwork == NULL )

		Create_Neural_Network();

	Add_Sensor_Neuron(ID,sensorID,sensorValueIndex,tau);
}

void ENVIRONMENT::Create_Synapse(void) {

        if ( neuralNetwork == NULL )

                Create_Neural_Network();

        neuralNetwork->Add_Synapse();
}

void ENVIRONMENT::Create_Touch_Sensor(int evalPeriod) {

        int objectIndex;

	int ID;

	std::cin >> ID;

        std::cin >> objectIndex;

        objects[objectIndex]->Create_Touch_Sensor(ID,evalPeriod);
}

void ENVIRONMENT::Create_Vestibular_Sensor(int evalPeriod) {

        int objectIndex;

        int ID;

        std::cin >> ID;

        std::cin >> objectIndex;

        objects[objectIndex]->Create_Vestibular_Sensor(ID,evalPeriod);
}

void ENVIRONMENT::Update_Sensor_Neurons(int timeStep) {

        for (int i=0;i<numberOfBodies;i++)

                objects[i]->Update_Sensor_Neurons(timeStep);

        for (int j=0;j<numberOfJoints;j++)

                joints[j]->Update_Sensor_Neurons(timeStep);
}
#endif
